@@ -66,8 +66,9 @@ void DocParser::pushContext()
   tokenizer.pushContext();
   contextStack.emplace();
   auto &ctx = contextStack.top();
-  ctx = context;
-  ctx.lineNo = tokenizer.getLineNr();
+  ctx           = context;
+  ctx.fileName  = tokenizer.getFileName();
+  ctx.lineNo    = tokenizer.getLineNr();
   context.token = tokenizer.token();
 }
 
@@ -75,6 +76,7 @@ void DocParser::popContext()
 {
   auto &ctx = contextStack.top();
   context = ctx;
+  tokenizer.setFileName(ctx.fileName);
   tokenizer.setLineNr(ctx.lineNo);
   contextStack.pop();
   tokenizer.popContext();
@@ -338,9 +340,9 @@ void DocParser::checkUnOrMultipleDocumentedParams()
         bool first=TRUE;
         QCString errMsg = "The following parameter";
         if (undocParams.size()>1) errMsg+="s";
-        errMsg+=" of "+
-            QCString(context.memberDef->qualifiedName()) +
-            QCString(argListToString(al)) +
+        errMsg+=QCString(" of ")+
+            context.memberDef->qualifiedName() +
+            argListToString(al) +
             (undocParams.size()>1 ? " are" : " is") + " not documented:\n";
         for (const Argument &a : undocParams)
         {
@@ -626,6 +628,7 @@ void DocParser::handleStyleEnter(DocNodeVariant *parent,DocNodeList &children,
   children.append<DocStyleChange>(this,parent,context.nodeStack.size(),s,tagName,TRUE,
                                   context.fileName,tokenizer.getLineNr(),attribs);
   context.styleStack.push(&children.back());
+  context.inCodeStyle = s==DocStyleChange::Style::Typewriter;
 }
 
 /*! Called when a style change ends. For instance a \</b\> command is
@@ -671,6 +674,10 @@ void DocParser::handleStyleLeave(DocNodeVariant *parent,DocNodeList &children,
           topStyleChange(context.styleStack).tagName(),FALSE);
     context.styleStack.pop();
   }
+  if (s==DocStyleChange::Style::Typewriter)
+  {
+    context.inCodeStyle = false;
+  }
 }
 
 /*! Called at the end of a paragraph to close all open style changes
@@ -800,11 +807,12 @@ void DocParser::handleLinkedWord(DocNodeVariant *parent,DocNodeList &children,bo
   bool ambig = false;
   FileDef *fd = findFileDef(Doxygen::inputNameLinkedMap,context.fileName,ambig);
   auto lang = context.lang;
+  bool inSeeBlock = context.inSeeBlock || context.inCodeStyle;
   //printf("handleLinkedWord(%s) context.context=%s\n",qPrint(context.token->name),qPrint(context.context));
   if (!context.insideHtmlLink &&
-      (resolveRef(context.context,context.token->name,context.inSeeBlock,&compound,&member,lang,TRUE,fd,TRUE)
+      (resolveRef(context.context,context.token->name,inSeeBlock,&compound,&member,lang,TRUE,fd,TRUE)
        || (!context.context.isEmpty() &&  // also try with global scope
-           resolveRef(QCString(),context.token->name,context.inSeeBlock,&compound,&member,lang,FALSE,nullptr,TRUE))
+           resolveRef(QCString(),context.token->name,inSeeBlock,&compound,&member,lang,FALSE,nullptr,TRUE))
       )
      )
   {
@@ -815,7 +823,7 @@ void DocParser::handleLinkedWord(DocNodeVariant *parent,DocNodeList &children,bo
       if (member->isObjCMethod())
       {
         bool localLink = context.memberDef ? member->getClassDef()==context.memberDef->getClassDef() : FALSE;
-        name = member->objCMethodName(localLink,context.inSeeBlock);
+        name = member->objCMethodName(localLink,inSeeBlock);
       }
       children.append<DocLinkedWord>(
             this,parent,name,
@@ -882,6 +890,23 @@ void DocParser::handleLinkedWord(DocNodeVariant *parent,DocNodeList &children,bo
           cd->anchor(),
           cd->briefDescriptionAsTooltip());
   }
+  else if (const RequirementIntf *req = RequirementManager::instance().find(name); req!=nullptr) // link to requirement
+  {
+    if (Config_getBool(GENERATE_REQUIREMENTS))
+    {
+      children.append<DocLinkedWord>(
+           this,parent,name,
+           QCString(), // link to local requirements overview also for external references
+           req->getOutputFileBase(),
+           req->id(),
+           req->title()
+          );
+    }
+    else // cannot link to a page that does not exist
+    {
+      children.append<DocWord>(this,parent,name);
+    }
+  }
   else // normal non-linkable word
   {
     AUTO_TRACE_ADD("non-linkable");
@@ -1193,6 +1218,42 @@ void DocParser::handleImage(DocNodeVariant *parent, DocNodeList &children)
   children.get_last<DocImage>()->parse();
 }
 
+void DocParser::handleIFile(char cmdChar,const QCString &cmdName)
+{
+  AUTO_TRACE();
+  Token tok=tokenizer.lex();
+  if (!tok.is(TokenRetval::TK_WHITESPACE))
+  {
+    warn_doc_error(context.fileName,tokenizer.getLineNr(),"expected whitespace after '{:c}{}' command",
+      cmdChar,cmdName);
+    return;
+  }
+  tokenizer.setStateFile();
+  tok=tokenizer.lex();
+  tokenizer.setStatePara();
+  if (!tok.is(TokenRetval::TK_WORD))
+  {
+    warn_doc_error(context.fileName,tokenizer.getLineNr(),"unexpected token {} as the argument of '{:c}{}'",
+      tok.to_string(),cmdChar,cmdName);
+    return;
+  }
+  context.fileName = context.token->name;
+  tokenizer.setStatePara();
+}
+
+void DocParser::handleILine(char cmdChar,const QCString &cmdName)
+{
+  AUTO_TRACE();
+  tokenizer.setStateILine();
+  Token tok = tokenizer.lex();
+  if (!tok.is(TokenRetval::TK_WORD))
+  {
+    warn_doc_error(context.fileName,tokenizer.getLineNr(),"invalid argument for command '{:c}{}'",
+      cmdChar,cmdName);
+    return;
+  }
+  tokenizer.setStatePara();
+}
 
 /* Helper function that deals with the most common tokens allowed in
  * title like sections.
@@ -1429,16 +1490,10 @@ reparsetoken:
           handleImage(parent,children);
           break;
         case CommandType::CMD_ILINE:
-          tokenizer.pushState();
-          tokenizer.setStateILine();
-          (void)tokenizer.lex();
-          tokenizer.popState();
+          handleILine(tok.command_to_char(),tokenName);
           break;
         case CommandType::CMD_IFILE:
-          tokenizer.pushState();
-          tokenizer.setStateIFile();
-          (void)tokenizer.lex();
-          tokenizer.popState();
+          handleIFile(tok.command_to_char(),tokenName);
           break;
         default:
           return FALSE;
@@ -1809,7 +1864,8 @@ static size_t skipToEndMarker(const char *data,size_t i,size_t len,const QCStrin
 QCString DocParser::processCopyDoc(const char *data,size_t &len)
 {
   AUTO_TRACE("data={} len={}",Trace::trunc(data),len);
-  GrowBuf buf;
+  QCString result;
+  result.reserve(len+32);
   size_t i=0;
   int lineNr = tokenizer.getLineNr();
   while (i<len)
@@ -1845,10 +1901,10 @@ QCString DocParser::processCopyDoc(const char *data,size_t &len)
             context.copyStack.push_back(def);
             auto addDocs = [&](const QCString &file_,int line_,const QCString &doc_)
             {
-              buf.addStr(" \\ifile \""+file_+"\" ");
-              buf.addStr("\\iline "+QCString().setNum(line_)+" \\ilinebr ");
+              result+=" \\ifile \""+file_+"\" ";
+              result+="\\iline "+QCString().setNum(line_)+" \\ilinebr ";
               size_t len_ = doc_.length();
-              buf.addStr(processCopyDoc(doc_.data(),len_));
+              result+=processCopyDoc(doc_.data(),len_);
             };
             if (isBrief)
             {
@@ -1863,12 +1919,12 @@ QCString DocParser::processCopyDoc(const char *data,size_t &len)
                 const ArgumentList &docArgList = md->templateMaster() ?
                     md->templateMaster()->argumentList() :
                     md->argumentList();
-                buf.addStr(inlineArgListToDoc(docArgList));
+                result+=inlineArgListToDoc(docArgList);
               }
             }
             context.copyStack.pop_back();
-            buf.addStr(" \\ilinebr \\ifile \""+context.fileName+"\" ");
-            buf.addStr("\\iline "+QCString().setNum(lineNr)+" ");
+            result+=" \\ilinebr \\ifile \""+context.fileName+"\" ";
+            result+="\\iline "+QCString().setNum(lineNr)+" ";
           }
           else
           {
@@ -1893,40 +1949,38 @@ QCString DocParser::processCopyDoc(const char *data,size_t &len)
         {
           size_t orgPos = i;
           i=skipToEndMarker(data,k,len,endMarker);
-          buf.addStr(data+orgPos,i-orgPos);
+          result+=QCString(data+orgPos,i-orgPos);
           // TODO: adjust lineNr
         }
         else
         {
-          buf.addChar(c);
+          result+=c;
           i++;
         }
       }
     }
     else // not a command, just copy
     {
-      buf.addChar(c);
+      result+=c;
       i++;
       lineNr += (c=='\n') ? 1 : 0;
     }
   }
-  len = buf.getPos();
-  buf.addChar(0);
-  AUTO_TRACE_EXIT("result={}",Trace::trunc(buf.get()));
-  return buf.get();
+  len = result.length();
+  AUTO_TRACE_EXIT("result={}",Trace::trunc(result));
+  return result;
 }
 
 
 //---------------------------------------------------------------------------
 
 IDocNodeASTPtr validatingParseDoc(IDocParser &parserIntf,
-                            const QCString &fileName,int startLine,
-                            const Definition *ctx,const MemberDef *md,
-                            const QCString &input,bool indexWords,
-                            bool isExample, const QCString &exampleName,
-                            bool singleLine, bool linkFromIndex,
-                            bool markdownSupport,
-                            bool autolinkSupport)
+                                  const QCString &fileName,
+                                  int startLine,
+                                  const Definition *ctx,
+                                  const MemberDef *md,
+                                  const QCString &input,
+                                  const DocOptions &options)
 {
   DocParser *parser = dynamic_cast<DocParser*>(&parserIntf);
   assert(parser!=nullptr);
@@ -1971,7 +2025,7 @@ IDocNodeASTPtr validatingParseDoc(IDocParser &parserIntf,
   parser->context.scope = ctx;
   parser->context.lang = getLanguageFromFileName(fileName);
 
-  if (indexWords && Doxygen::searchIndex.enabled())
+  if (options.indexWords() && Doxygen::searchIndex.enabled())
   {
     if (md)
     {
@@ -1990,8 +2044,8 @@ IDocNodeASTPtr validatingParseDoc(IDocParser &parserIntf,
   }
 
   parser->context.fileName = fileName;
-  parser->context.relPath = (!linkFromIndex && ctx) ?
-               QCString(relativePathToRoot(ctx->getOutputFileBase())) :
+  parser->context.relPath = (!options.linkFromIndex() && ctx) ?
+               relativePathToRoot(ctx->getOutputFileBase()) :
                QCString("");
   //printf("ctx->name=%s relPath=%s\n",qPrint(ctx->name()),qPrint(parser->context.relPath));
   parser->context.memberDef = md;
@@ -1999,21 +2053,23 @@ IDocNodeASTPtr validatingParseDoc(IDocParser &parserIntf,
   while (!parser->context.styleStack.empty()) parser->context.styleStack.pop();
   while (!parser->context.initialStyleStack.empty()) parser->context.initialStyleStack.pop();
   parser->context.inSeeBlock = FALSE;
+  parser->context.inCodeStyle = FALSE;
   parser->context.xmlComment = FALSE;
   parser->context.insideHtmlLink = FALSE;
   parser->context.includeFileText = "";
   parser->context.includeFileOffset = 0;
   parser->context.includeFileLength = 0;
-  parser->context.isExample = isExample;
-  parser->context.exampleName = exampleName;
+  parser->context.isExample = options.isExample();
+  parser->context.exampleName = options.exampleName();
   parser->context.hasParamCommand = FALSE;
   parser->context.hasReturnCommand = FALSE;
   parser->context.retvalsFound.clear();
   parser->context.paramsFound.clear();
-  parser->context.markdownSupport = markdownSupport;
-  parser->context.autolinkSupport = autolinkSupport;
+  parser->context.markdownSupport = options.markdownSupport();
+  parser->context.autolinkSupport = options.autolinkSupport();
 
   //printf("Starting comment block at %s:%d\n",qPrint(parser->context.fileName),startLine);
+  parser->tokenizer.setFileName(fileName);
   parser->tokenizer.setLineNr(startLine);
   size_t ioLen = input.length();
   QCString inpStr = parser->processCopyDoc(input.data(),ioLen);
@@ -2021,12 +2077,12 @@ IDocNodeASTPtr validatingParseDoc(IDocParser &parserIntf,
   {
     inpStr+='\n';
   }
-  //printf("processCopyDoc(in='%s' out='%s')\n",input,qPrint(inpStr));
+  //printf("processCopyDoc(in='%s' out='%s')\n",qPrint(input),qPrint(inpStr));
   parser->tokenizer.init(inpStr.data(),parser->context.fileName,
                          parser->context.markdownSupport,parser->context.insideHtmlLink);
 
   // build abstract syntax tree
-  auto ast = std::make_unique<DocNodeAST>(DocRoot(parser,md!=nullptr,singleLine));
+  auto ast = std::make_unique<DocNodeAST>(DocRoot(parser,md!=nullptr,options.singleLine()));
   std::get<DocRoot>(ast->root).parse();
 
   if (Debug::isFlagSet(Debug::PrintTree))
@@ -2069,6 +2125,7 @@ IDocNodeASTPtr validatingParseTitle(IDocParser &parserIntf,const QCString &fileN
   while (!parser->context.styleStack.empty()) parser->context.styleStack.pop();
   while (!parser->context.initialStyleStack.empty()) parser->context.initialStyleStack.pop();
   parser->context.inSeeBlock = FALSE;
+  parser->context.inCodeStyle = FALSE;
   parser->context.xmlComment = FALSE;
   parser->context.insideHtmlLink = FALSE;
   parser->context.includeFileText = "";
@@ -2122,6 +2179,7 @@ IDocNodeASTPtr validatingParseText(IDocParser &parserIntf,const QCString &input)
   while (!parser->context.styleStack.empty()) parser->context.styleStack.pop();
   while (!parser->context.initialStyleStack.empty()) parser->context.initialStyleStack.pop();
   parser->context.inSeeBlock = FALSE;
+  parser->context.inCodeStyle = FALSE;
   parser->context.xmlComment = FALSE;
   parser->context.insideHtmlLink = FALSE;
   parser->context.includeFileText = "";
@@ -2168,6 +2226,7 @@ IDocNodeASTPtr createRef(IDocParser &parserIntf,const QCString &target,const QCS
   if (!srcFile.isEmpty())
   {
     parser->context.fileName = srcFile;
+    parser->tokenizer.setFileName(srcFile);
     parser->tokenizer.setLineNr(srcLine);
   }
   return std::make_unique<DocNodeAST>(DocRef(parser,nullptr,target,context));
